==> osam-6.3.3.tar.gz: lib/src/domain/property_notifier.dart <==
import 'package:flutter/foundation.dart';
import 'package:hive/hive.dart';
import '../../osam.dart';

part 'property_notifier.g.dart';

typedef Update<T> = Function(T value);

abstract class IProperty<T> implements ValueListenable<T> {
  factory IProperty(T value) => _PropertyNotifier(value);

  @override
  T get value;
}

@HiveType(typeId: 223)
class _PropertyNotifier<T> extends ChangeNotifier
    with EquatableMixin, IProperty<T> {
  _PropertyNotifier(this._value) {
    _lastKnownHashcode = _ComparableWrapper(_value).hashCode;
  }

  @HiveField(0)
  T _value;

  int _lastKnownHashcode;

  set value(T newValue) {
    final newHashcode = _ComparableWrapper(newValue).hashCode;
    if (_lastKnownHashcode != newHashcode) {
      _value = newValue;
      _lastKnownHashcode = newHashcode;
      notifyListeners();
    }
  }

  @override
  T get value => _value;

  void update(Update<T> update) {
    update(_value);
    final newHashcode = _ComparableWrapper(_value).hashCode;
    if (_lastKnownHashcode != newHashcode) {
      _lastKnownHashcode = newHashcode;
      notifyListeners();
    }
  }

  @override
  List<Object> get props => [_value];
}

class _ComparableWrapper<V> extends Equatable {
  final V _value;

  _ComparableWrapper(this._value);

  @override
  List<Object> get props => [_value];
}

class PropertyChanger {
  @protected
  void set<T>(IProperty<T> notifier, T value) =>
      (notifier as _PropertyNotifier).value = value;
  @protected
  void update<T>(IProperty<T> notifier, Update<T> update) =>
      (notifier as _PropertyNotifier).update(update);
}

==> osam-6.3.4.tar.gz: lib/src/domain/property_notifier.dart <==
import 'package:flutter/foundation.dart';
import 'package:hive/hive.dart';
import '../../osam.dart';

part 'property_notifier.g.dart';

typedef Update<T> = Function(T value);

abstract class IProperty<T> implements ValueListenable<T> {
  factory IProperty(T value) => _PropertyNotifier<T>(value);

  @override
  T get value;
}

@HiveType(typeId: 223)
class _PropertyNotifier<T> extends ChangeNotifier
    with EquatableMixin, IProperty<T> {
  _PropertyNotifier(this._value) {
    _lastKnownHashcode = _ComparableWrapper(_value).hashCode;
  }

  @HiveField(0)
  T _value;

  int _lastKnownHashcode;

  set value(T newValue) {
    final newHashcode = _ComparableWrapper(newValue).hashCode;
    if (_lastKnownHashcode != newHashcode) {
      _value = newValue;
      _lastKnownHashcode = newHashcode;
      notifyListeners();
    }
  }

  @override
  T get value => _value;

  void update(Update<T> update) {
    update(_value);
    final newHashcode = _ComparableWrapper(_value).hashCode;
    if (_lastKnownHashcode != newHashcode) {
      _lastKnownHashcode = newHashcode;
      notifyListeners();
    }
  }

  @override
  List<Object> get props => [_value];
}

class _ComparableWrapper<V> extends Equatable {
  final V _value;

  _ComparableWrapper(this._value);

  @override
  List<Object> get props => [_value];
}

class PropertyChanger {
  @protected
  void set<T>(IProperty<T> notifier, T value) =>
      (notifier as _PropertyNotifier<T>).value = value;
  @protected
  void update<T>(IProperty<T> notifier, Update<T> update) =>
      (notifier as _PropertyNotifier<T>).update(update);
}
